START TRANSACTION ;

-- --------------------------------------------------------
# Добавить колонку Degree в зачетки для упрощения кода миграции
ALTER TABLE `record_books`
  ADD COLUMN `Degree` VARCHAR(10),
  ADD KEY (`Degree`);

-- --------------------------------------------------------
# Заполнить таблицу record_books фейковыми значениями следующего вида:
# fake-<degree>-<studentId>
# <degree> - уровень подготовки (b - бакалавр, m - магистр)
# <studentId> - идентификатор студента
# Предварительно нужно создать view_students, если ее еще нет.
CREATE OR REPLACE VIEW `view_students` AS
  SELECT  students.ID         AS 'StudentID',
    accounts.LastName,
    accounts.FirstName,
    accounts.SecondName,
    students.AccountID,
    students_groups.SemesterID,
    view_groups_reduced.*
  FROM `students`
    LEFT JOIN `accounts`            ON accounts.ID = students.AccountID
    INNER JOIN `students_groups`     ON students_groups.StudentID = students.ID
    LEFT JOIN `view_groups_reduced` ON view_groups_reduced.GroupID = students_groups.GroupID;


DELIMITER //

CREATE PROCEDURE RecordBooks_Fill()
BEGIN
  DECLARE done INT DEFAULT FALSE;
  DECLARE vStudentID INT;
  DECLARE vDegree VARCHAR(10);
  DECLARE vRecordBookNum VARCHAR(50);
  DECLARE cur CURSOR FOR
    SELECT view_students.StudentID AS StudentID, view_students.Degree AS Degree FROM `view_students`
    GROUP BY view_students.Degree, view_students.StudentID;
  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

  OPEN cur;
  read_loop: LOOP
    FETCH cur INTO vStudentID, vDegree;
    IF done THEN
      LEAVE read_loop;
    END IF;
    SET vRecordBookNum = REPLACE(REPLACE("fake-<degree>-<studentid>", "<degree>", vDegree),
                                                                 "<studentid>", vStudentID);
    INSERT `record_books` (`StudentID`, `ExternalID`, `PlanID`, `Degree`)
      VALUES (vStudentID, vRecordBookNum, 0, vDegree);
  END LOOP;
  CLOSE cur;
END //

DELIMITER ;

CALL RecordBooks_Fill();
DROP PROCEDURE RecordBooks_Fill;

COMMIT ;

START TRANSACTION;

-- --------------------------------------------------------
# groups_students
# 1. создать поле RecordBookID
# 2. заполнить поле RecordBookID
# Удаление старых связей и создание новых после всех использований этой таблицы

ALTER TABLE `students_groups`
  ADD COLUMN `RecordBookID` int(11),
  ADD KEY  (`RecordBookID`);

COMMIT;

START TRANSACTION;
UPDATE `students_groups`
  INNER JOIN study_groups on study_groups.ID = students_groups.GroupID
  INNER JOIN grades on grades.ID = study_groups.GradeID
  INNER JOIN record_books on
                            record_books.Degree = grades.Degree and
                            record_books.StudentID = students_groups.StudentID
SET students_groups.RecordBookID = record_books.ID;

COMMIT;

START TRANSACTION ;
-- --------------------------------------------------------
# rating_table
# 1. создать поле RecordBookID
# 2. заполнить поле RecordBookID
# 3. удалить все связи поля StudentID и потом само поле
# 4. создать связи для RecordBookID

ALTER TABLE `rating_table`
  ADD COLUMN `RecordBookID` int(11),
  ADD KEY (`RecordBookID`);

COMMIT ;

START TRANSACTION;

UPDATE rating_table
  INNER JOIN submodules on submodules.ID = rating_table.SubmoduleID
  INNER JOIN modules on modules.ID = submodules.ModuleID
  INNER JOIN disciplines on disciplines.ID = modules.DisciplineID
  INNER JOIN students_groups on students_groups.SemesterID = disciplines.SemesterID
                                AND students_groups.StudentID = rating_table.StudentID
SET rating_table.RecordBookID = students_groups.RecordBookID;

ALTER TABLE `rating_table`
  DROP FOREIGN KEY `rating_table_ibfk_1`,
  DROP INDEX `PRIMARY`,
  DROP INDEX `StudentID`;
ALTER TABLE `rating_table`
  DROP COLUMN `StudentID`;

ALTER TABLE `rating_table`
  ADD PRIMARY KEY (`RecordBookID`, `SubmoduleID`),
  ADD CONSTRAINT `rating_table_ibfk_1` FOREIGN KEY (`RecordBookID`) REFERENCES `record_books` (`ID`);

COMMIT;

START TRANSACTION;

# exam_period_options
# 1. создать поле RecordBookID

ALTER TABLE `exam_period_options`
  ADD COLUMN `RecordBookID` int(11),
  ADD KEY (`RecordBookID`);

# discipline_students
# 1. создать поле RecordBookID

ALTER TABLE `disciplines_students`
  ADD COLUMN `RecordBookID` int(11),
  ADD KEY (`RecordBookID`);

COMMIT;

START TRANSACTION;

-- --------------------------------------------------------
# exam_period_options
# 1. создать поле RecordBookID - done
# 2. заполнить поле RecordBookID
# 3. удалить все связи поля StudentID и потом само поле
# 4. создать связи для поля RecordBookID

UPDATE exam_period_options
  INNER JOIN submodules on submodules.ID = exam_period_options.SubmoduleID
  INNER JOIN modules on modules.ID = submodules.ModuleID
  INNER JOIN disciplines on disciplines.ID = modules.DisciplineID
  INNER JOIN students_groups on students_groups.SemesterID = disciplines.SemesterID
                                AND students_groups.StudentID = exam_period_options.StudentID
SET exam_period_options.RecordBookID = students_groups.RecordBookID;

ALTER TABLE `exam_period_options`
  DROP FOREIGN KEY `exam_period_options_ibfk_2`,
  DROP INDEX `StudentID`,
  DROP INDEX `StudentID_2`,
  DROP INDEX `StudentID_3`,
  DROP FOREIGN KEY `exam_period_options_ibfk_1`,
  DROP INDEX `SubmoduleID`,
  DROP INDEX `SubmoduleID_2`,
  DROP COLUMN `StudentID`;

ALTER TABLE `exam_period_options`
  ADD KEY (`SubmoduleID`),
  ADD CONSTRAINT `Unique` UNIQUE (`RecordBookID`, `SubmoduleID`),
  ADD CONSTRAINT `exam_period_options_ibfk_1` FOREIGN KEY (`SubmoduleID`) REFERENCES `submodules` (`ID`),
  ADD CONSTRAINT `exam_period_options_ibfk_2` FOREIGN KEY (`RecordBookID`) REFERENCES `record_books` (`ID`);


-- --------------------------------------------------------
# discipline_students
# 1. создать поле RecordBookID - done
# 2. заполнить поле RecordBookID
# 3. удалить все связи поля StudentID и потом само поле
# 4. создать связи для поля RecordBookID


UPDATE `disciplines_students`
  INNER JOIN disciplines on disciplines.ID = disciplines_students.DisciplineID
  INNER JOIN students_groups on students_groups.SemesterID = disciplines.SemesterID
                                AND students_groups.StudentID = disciplines_students.StudentID
SET disciplines_students.RecordBookID = students_groups.RecordBookID;

ALTER TABLE `disciplines_students`
  DROP FOREIGN KEY `disciplines_students_ibfk_2`,
  DROP INDEX DisciplineID_2,
  DROP INDEX `StudentID`,
  DROP COLUMN `StudentID`;

ALTER TABLE `disciplines_students`
  ADD CONSTRAINT `Unique` UNIQUE (`RecordBookID`, `DisciplineID`),
  ADD CONSTRAINT `disciplines_students_ibfk_2` FOREIGN KEY (`RecordBookID`) REFERENCES `record_books` (`ID`);

COMMIT;

START TRANSACTION;
-- --------------------------------------------------------
# logs_bind_students
# 1. создать поле RecordBookID
# 2. заполнить поле RecordBookID
# 3. удалить все связи поля StudentID и потом само поле
# 4. создать связи для поля RecordBookID

ALTER TABLE `logs_binds_students`
  ADD COLUMN `RecordBookID` int(11),
  ADD KEY (`RecordBookID`);

UPDATE `logs_binds_students`
  INNER JOIN disciplines on disciplines.ID = logs_binds_students.DisciplineID
  INNER JOIN students_groups on students_groups.SemesterID = disciplines.SemesterID
                                AND students_groups.StudentID = logs_binds_students.StudentID
SET logs_binds_students.RecordBookID = students_groups.RecordBookID;

ALTER TABLE `logs_binds_students`
  DROP FOREIGN KEY `logs_binds_students_ibfk_3`,
  DROP INDEX `StudentID`,
  DROP COLUMN `StudentID`;

ALTER TABLE `logs_binds_students`
  ADD CONSTRAINT `logs_binds_students_ibfk_3` FOREIGN KEY (`RecordBookID`) REFERENCES `record_books` (`ID`);

COMMIT;

START TRANSACTION;

# logs_rating
# 1. создать поле RecordBookID и связи для него
ALTER TABLE `logs_rating`
  ADD COLUMN `RecordBookID` int(11),
  ADD KEY (`RecordBookID`);

COMMIT;

START TRANSACTION;
-- --------------------------------------------------------
# logs_rating
# 1. создать поле RecordBookID - done
# 2. заполнить поле RecordBookID
# 3. удалить все связи поля StudentID и потом само поле
# 4. создать связи для поля RecordBookID

UPDATE `logs_rating`
  INNER JOIN submodules on submodules.ID = logs_rating.SubmoduleID
  INNER JOIN modules on modules.ID = submodules.ModuleID
  INNER JOIN disciplines on disciplines.ID = modules.DisciplineID
  INNER JOIN students_groups on students_groups.SemesterID = disciplines.SemesterID
                                AND students_groups.StudentID = logs_rating.StudentID
SET logs_rating.RecordBookID = students_groups.RecordBookID;

ALTER TABLE `logs_rating`
  DROP FOREIGN KEY `logs_rating_ibfk_1`,
  DROP INDEX `StudentID`,
  DROP COLUMN `StudentID`;

ALTER TABLE `logs_rating`
  ADD CONSTRAINT `logs_rating_ibfk_1` FOREIGN KEY (`RecordBookID`) REFERENCES `record_books` (`ID`);

COMMIT;

START TRANSACTION ;

# 3. удалить все связи поля StudentID и потом само поле
# 4. создать связи для поля RecordBookID
DROP VIEW `view_students`;

ALTER TABLE `students_groups`
  DROP FOREIGN KEY `students_groups_ibfk_1`,
  DROP INDEX `StudentID`,
  DROP INDEX `StudentID_2`,
  DROP COLUMN `StudentID`;

ALTER TABLE `students_groups`
  ADD CONSTRAINT `students_groups_ibfk_1` FOREIGN KEY (`RecordBookID`) REFERENCES `record_books` (`ID`),
  ADD UNIQUE `SemesterID_RecordBookID` (`SemesterID`, `RecordBookID`);

# Удалить временную колонку Degree в зачетках
ALTER TABLE `record_books`
  DROP INDEX `Degree`,
  DROP COLUMN `Degree`;

COMMIT ;

DROP PROCEDURE IF EXISTS GetDepartments;